#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>

#include <config.h>
#include "options.h"
#include "error.h"


bool get_cmdine_opts(int argc, char** argv)
{
	int c;

	/* initialize global option vaiables */
	opt_filename = NULL;
	opt_output = NULL;
	opt_line = 0;
	opt_column = 0;
	opt_listmembers = NULL;
	opt_liststatic = NULL;
	opt_lang = "c++";
	opt_cache = 1;
	opt_output = NULL;

	/* the struct option look like:
	   struct option { const char *name; int has_arg; int *flag; int val; }; */
	static struct option long_options[] = {
		{"cache"        , 1 , 0 , 'a' }  , 
		{"column"       , 1 , 0 , 'c' }  , 
		{"help"         , 0 , 0 , 'h' }  , 
		{"line"         , 1 , 0 , 'l' }  , 
		{"list-members" , 1 , 0 , 'm' }  , 
		{"output"       , 1 , 0 , 'o' }  , 
		//{"lang"         , 1 , 0 , 'g' }  , 
		{"version"      , 0 , 0 , 'v' }  , 
		{0              , 0 , 0 , 0   } 
	};

    while (1)
	{
/*         int this_option_optind = optind ? optind : 1; */
        int option_index = 0;
        c = getopt_long (argc, argv, "a:c:hl:m:o:s:v", long_options, &option_index);
        if (c == -1)
            break;

        switch (c)
		{
        case 0:
            printf ("option %s", long_options[option_index].name);
            if (optarg)
                printf (" with arg %s", optarg);
            printf ("\n");
            break;

        case 'a':
			opt_cache = atoi(optarg);
            break;

        case 'c':
			opt_column = atoi(optarg);
            break;

        case 'l':
			opt_line = atoi(optarg);
            break;

        case 'g':
			opt_lang = optarg;
            break;

        case 'h':
			usage();
            break;

        case 'm':
			opt_listmembers = optarg;
            break;

        case 'o':
			opt_output = optarg;
            break;

        case 'v':
			printf(PACKAGE_NAME "\nVersion: " VERSION "\n"); 
            break;

        case '?':
            break;

        default:
            printf ("?? getopt returned character code 0%o ??\n", c);
        }
    }

	// file name given
    if (optind < argc)
	{
		opt_filename = argv[optind];
		optind++;

        if (optind < argc)
			bailout("Only one source file allowed.\nSee icomplete --help for a usage description.");
    }
	// read from stdin
	else
	{
	}

    return true;
}

void read_config()
{
	/* realloc the vector every INCREMENT count */
	const int INCREMENT = 10;
	/* cur_section points to the vector for the current [section] of the config file */
	vector* cur_section = NULL;

	config.cpp_includes.count = 0;
	config.cpp_includes.entries = NULL;
	config.cpp_macros.count = 0;
	config.cpp_includes.entries = NULL;

	char* homedir = getenv("HOME");
	char homeconfig[512];
	if (homedir)
		snprintf(homeconfig, 512, "%s/.icompleterc", homedir);

	char *configs[3];
	configs[0] = ".icomplete";
	configs[1] = homedir ? homeconfig : NULL;
	configs[2] = SYSCONFDIR "/icomplete.conf";

	char line[512];
	char *ptr = NULL;
	for (int i=0; i < 3; i++)
	{
		if (configs[i] == NULL)
			continue;

		FILE *file = fopen(configs[i], "r");
		if (file)
		{
			while (fgets(line, 512, file) != NULL)
			{
				ptr = line;
				if (!strcmp(line, "[cpp_include_paths]\n"))
				{
					cur_section = &config.cpp_includes;
					continue;
				}
				else if (!strcmp(line, "[cpp_macros]\n"))
				{
					cur_section = &config.cpp_macros;
					continue;
				}

				/* strip leading whitespace and remove comments and empty lines */
				while (*ptr == ' ' || *ptr == '\t')
					ptr++;
				if (*ptr == '#')
					continue;
				if (*ptr == '\n')
					continue;

				if (cur_section != NULL)
				{
					if (cur_section->count % INCREMENT == 0)
					{
						cur_section->entries = realloc (cur_section->entries, sizeof(char*)*(cur_section->count + INCREMENT));
						if (cur_section->entries == NULL)
							bailout("realloc in read_config() failed");
					}
					/* remove trailing \n */
					int l = strlen(ptr);
					if (l && ptr[l-1] == '\n')
						ptr[l-1] = '\0';
					char *dup = strdup(line);
					if (!dup)
						bailout ("strdup in read_config() failed");
					cur_section->entries[cur_section->count] = dup;
					cur_section->count++;
#if DEBUG >= 2
					printf("DEBUG: option found: %s\n", dup);
#endif
				}
			}
		}
	}
/*        free(config.cpp_includes.entries[0]);*/
}

void free_config()
{
#if DEBUG >= 2
	printf("DEBUG: freeing config\n");
#endif
	if (config.cpp_includes.entries != NULL)
	{
		for (int i=0; i < config.cpp_includes.count; i++)
			free(config.cpp_includes.entries[i]);
		free(config.cpp_includes.entries);
		config.cpp_includes.entries = NULL;
	}

	if (config.cpp_macros.entries != NULL)
	{
		for (int i=0; i < config.cpp_macros.count; i++)
			free(config.cpp_macros.entries[i]);
		free(config.cpp_macros.entries);
		config.cpp_macros.entries = NULL;
	}
}

