#ifndef __TREE_H__
#define __TREE_H__

#include <stdio.h>

#include "parse.h"

/* this struct holds the inheritance information */
typedef struct Tree
{
	char *name;		        // the name of the class
	int size;               // number of valid elements in the 'inherits' member
	struct Tree **inherits; // pointers to next elements in the tree
} Tree;

typedef enum
{
	NOT_FOUND = 0,
	DIRECT_HIT,
	SUBCLASS
} Match;

/*
 * Build an inheritance tree for classes
 * Looks like:
 *
 *             --- subclass1
 * rootclass <
 *             --- subclass2 -- subclass2_1 --> subclass3
 *                            \ subclass2_2
 *
 * Caller must free the tree with
 * free_tree() after calling this function
 */
Tree* build_inheritance_tree(const char *class);
/* same as build_inheritance_tree() but for #include files */
Tree* build_include_tree(Tree* start, const char *filename);

/* free()'s all memory allocated of a Tree* */
void free_tree(Tree *tree);

/* NULL pointer checks must be performed by the caller */
void write_tree_to_file(Tree *tree, FILE* file);

/*
 * This function searches a given Tree for a class name
 * and returns if the class is the root element of the tree
 * (=DIRECT_HIT), or a subclass.
 */
Match find_in_tree(const Tree *tree, const char *class);

/* when we find a tag, the following conditions must be met, to show the tag:
 * - method must be part of a class (key=class)
 * - access permissions (private, protected, public) must match.
 * - FIXME: method must not be pure virtual
 * - no static method
 */
bool is_member_of_scope (const tagEntry *entry, const Scope *scope, const Tree* tree);

/**
 * parses a tags file to show all class members
 * relative to the scope of another class
 */
void find_entries (const Expression *exp, const Scope *scope);

#endif /* __TREE_H__ */
